@@ -1,25 +1,27 @@
 use std::{io, sync::Arc, time::Duration};
 
 use arboard::Clipboard;
-use crossterm::{
-    self, cursor,
-    event::{Event, EventStream, KeyCode, KeyEvent, KeyEventKind, KeyEventState, KeyModifiers},
-    execute,
-    style::Color,
-    terminal::{self, disable_raw_mode, enable_raw_mode},
-};
 use futures::StreamExt;
-use futures_timer::Delay;
-use promkit::{listbox, style::StyleBuilder, text, text_editor, PaneFactory};
+use promkit_core::{
+    crossterm::{
+        cursor,
+        event::{Event, EventStream, KeyCode, KeyEvent, KeyEventKind, KeyEventState, KeyModifiers},
+        execute,
+        style::{Color, ContentStyle},
+        terminal::{self, disable_raw_mode, enable_raw_mode},
+    },
+    PaneFactory,
+};
+use promkit_widgets::text::{self, Text};
 use tokio::{
     sync::{mpsc, Mutex, RwLock},
     task::JoinHandle,
 };
 
 use crate::{
-    Context, ContextMonitor, Editor, EditorTheme, IncrementalSearcher, PaneIndex, Processor,
-    Renderer, SearchProvider, SpinnerSpawner, ViewInitializer, ViewProvider, Visualizer,
-    EMPTY_PANE,
+    config::{event::Matcher, Keybinds, ReactivityControl},
+    Context, ContextMonitor, Editor, PaneIndex, Processor, Renderer, SearchProvider,
+    SpinnerSpawner, ViewInitializer, ViewProvider, Visualizer, EMPTY_PANE,
 };
 
 fn spawn_debouncer<T: Send + 'static>(
@@ -29,10 +31,8 @@ fn spawn_debouncer<T: Send + 'static>(
 ) -> tokio::task::JoinHandle<()> {
     tokio::spawn(async move {
         let mut last_query = None;
+        let mut delay = tokio::time::interval(duration);
         loop {
-            let delay = Delay::new(duration);
-            futures::pin_mut!(delay);
-
             tokio::select! {
                 maybe_query = debounce_rx.recv() => {
                     if let Some(query) = maybe_query {
@@ -41,7 +41,7 @@ fn spawn_debouncer<T: Send + 'static>(
                         break;
                     }
                 },
-                _ = delay => {
+                _ = delay.tick() => {
                     if let Some(text) = last_query.take() {
                         let _ = last_tx.send(text).await;
                     }
@@ -55,20 +55,32 @@ fn copy_to_clipboard(content: &str) -> text::State {
     match Clipboard::new() {
         Ok(mut clipboard) => match clipboard.set_text(content) {
             Ok(_) => text::State {
-                text: "Copied to clipboard".to_string(),
-                style: StyleBuilder::new().fgc(Color::Green).build(),
+                text: Text::from("Copied to clipboard"),
+                style: ContentStyle {
+                    foreground_color: Some(Color::Green),
+                    ..Default::default()
+                },
+                ..Default::default()
             },
             Err(e) => text::State {
-                text: format!("Failed to copy to clipboard: {}", e),
-                style: StyleBuilder::new().fgc(Color::Red).build(),
+                text: Text::from(format!("Failed to copy to clipboard: {}", e)),
+                style: ContentStyle {
+                    foreground_color: Some(Color::Red),
+                    ..Default::default()
+                },
+                ..Default::default()
             },
         },
         // arboard fails (in the specific environment like linux?) on Clipboard::new()
         // suppress the errors (but still show them) not to break the prompt
         // https://github.com/1Password/arboard/issues/153
         Err(e) => text::State {
-            text: format!("Failed to setup clipboard: {}", e),
-            style: StyleBuilder::new().fgc(Color::Red).build(),
+            text: Text::from(format!("Failed to setup clipboard: {}", e)),
+            style: ContentStyle {
+                foreground_color: Some(Color::Red),
+                ..Default::default()
+            },
+            ..Default::default()
         },
     }
 }
@@ -81,39 +93,24 @@ enum Focus {
 #[allow(clippy::too_many_arguments)]
 pub async fn run<T: ViewProvider + SearchProvider>(
     item: &'static str,
-    spin_duration: Duration,
-    query_debounce_duration: Duration,
-    resize_debounce_duration: Duration,
+    reactivity_control: ReactivityControl,
     provider: &mut T,
-    text_editor_state: text_editor::State,
-    editor_focus_theme: EditorTheme,
-    editor_defocus_theme: EditorTheme,
-    listbox_state: listbox::State,
-    search_result_chunk_size: usize,
-    search_load_chunk_size: usize,
+    editor: Editor,
+    loading_suggestions_task: JoinHandle<anyhow::Result<()>>,
     no_hint: bool,
+    keybinds: Keybinds,
 ) -> anyhow::Result<()> {
     enable_raw_mode()?;
     execute!(io::stdout(), cursor::Hide)?;
 
     let size = terminal::size()?;
 
-    let searcher = IncrementalSearcher::new(listbox_state, search_result_chunk_size);
-    let loading_suggestions_task = searcher.spawn_load_task(provider, item, search_load_chunk_size);
-    let editor = Editor::new(
-        text_editor_state,
-        searcher,
-        editor_focus_theme,
-        editor_defocus_theme,
-    );
-
     let shared_renderer = Arc::new(Mutex::new(Renderer::try_init_draw(
         [
             editor.create_editor_pane(size.0, size.1),
             EMPTY_PANE.to_owned(),
             EMPTY_PANE.to_owned(),
             EMPTY_PANE.to_owned(),
-            EMPTY_PANE.to_owned(),
         ],
         no_hint,
     )?));
@@ -122,16 +119,26 @@ pub async fn run<T: ViewProvider + SearchProvider>(
 
     let (last_query_tx, mut last_query_rx) = mpsc::channel(1);
     let (debounce_query_tx, debounce_query_rx) = mpsc::channel(1);
-    let query_debouncer =
-        spawn_debouncer(debounce_query_rx, last_query_tx, query_debounce_duration);
+    let query_debouncer = spawn_debouncer(
+        debounce_query_rx,
+        last_query_tx,
+        reactivity_control.query_debounce_duration,
+    );
+    if !editor.text().is_empty() {
+        debounce_query_tx.send(editor.text()).await?;
+    }
 
     let (last_resize_tx, mut last_resize_rx) = mpsc::channel::<(u16, u16)>(1);
     let (debounce_resize_tx, debounce_resize_rx) = mpsc::channel(1);
-    let resize_debouncer =
-        spawn_debouncer(debounce_resize_rx, last_resize_tx, resize_debounce_duration);
+    let resize_debouncer = spawn_debouncer(
+        debounce_resize_rx,
+        last_resize_tx,
+        reactivity_control.resize_debounce_duration,
+    );
 
     let spinner_spawner = SpinnerSpawner::new(ctx.clone());
-    let spinning = spinner_spawner.spawn_spin_task(shared_renderer.clone(), spin_duration);
+    let spinning =
+        spinner_spawner.spawn_spin_task(shared_renderer.clone(), reactivity_control.spin_duration);
 
     let mut focus = Focus::Editor;
     let (editor_event_tx, mut editor_event_rx) = mpsc::channel::<Event>(1);
@@ -147,7 +154,13 @@ pub async fn run<T: ViewProvider + SearchProvider>(
     let processor = Processor::new(ctx.clone());
     let context_monitor = ContextMonitor::new(ctx.clone());
     let initializer = ViewInitializer::new(ctx.clone());
-    let initializing = initializer.initialize(provider, item, size, shared_renderer.clone());
+    let initializing = initializer.initialize(
+        provider,
+        item,
+        size,
+        shared_renderer.clone(),
+        keybinds.on_json_viewer,
+    );
 
     let main_task: JoinHandle<anyhow::Result<()>> = {
         let mut stream = EventStream::new();
@@ -160,12 +173,7 @@ pub async fn run<T: ViewProvider + SearchProvider>(
                             Event::Resize(width, height) => {
                                 debounce_resize_tx.send((width, height)).await?;
                             },
-                            Event::Key(KeyEvent {
-                                code: KeyCode::Char('c'),
-                                modifiers: KeyModifiers::CONTROL,
-                                kind: KeyEventKind::Press,
-                                state: KeyEventState::NONE,
-                            }) => {
+                            event if keybinds.exit.matches(&event) => {
                                 break 'main
                             },
                             Event::Key(KeyEvent {
@@ -188,8 +196,12 @@ pub async fn run<T: ViewProvider + SearchProvider>(
                                 } else {
                                     let size = terminal::size()?;
                                     pane = text::State {
-                                        text: "Failed to copy while rendering is in progress.".to_string(),
-                                        style: StyleBuilder::new().fgc(Color::Yellow).build(),
+                                        text: Text::from("Failed to copy while rendering is in progress.".to_string()),
+                                        style: ContentStyle {
+                                            foreground_color: Some(Color::Yellow),
+                                            ..Default::default()
+                                        },
+                                        ..Default::default()
                                     }.create_pane(size.0, size.1);
                                 }
                                 {
@@ -218,8 +230,12 @@ pub async fn run<T: ViewProvider + SearchProvider>(
                                         } else {
                                             let size = terminal::size()?;
                                             pane = text::State {
-                                                text: "Failed to switch pane while rendering is in progress.".to_string(),
-                                                style: StyleBuilder::new().fgc(Color::Yellow).build(),
+                                                text: Text::from("Failed to switch pane while rendering is in progress.".to_string()),
+                                                style: ContentStyle {
+                                                    foreground_color: Some(Color::Yellow),
+                                                    ..Default::default()
+                                                },
+                                                ..Default::default()
                                             }.create_pane(size.0, size.1);
                                         }
                                         {
